package ch.zhaw.gartenverwaltung.io;

import ch.zhaw.gartenverwaltung.types.Task;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.fasterxml.jackson.datatype.jsr310.deser.MonthDayDeserializer;
import javafx.util.converter.LocalDateStringConverter;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.time.LocalDate;
import java.time.MonthDay;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;

public class JsonTaskDatabase implements TaskDatabase{
    private final URL dataSource = getClass().getResource("taskdb.json");

    public static void main(String[] args) {
        Date date = new Date();
        Date yesterday = new Date(-1);
        Task testTask = new Task("water", "apply water", date);
        JsonTaskDatabase jsonTaskDatabase = new JsonTaskDatabase();
        try {
            jsonTaskDatabase.saveTask(testTask);
            System.out.println(jsonTaskDatabase.getTaskList(yesterday,date));
        } catch (Exception e){
            System.out.println("Task saving failed!");
        }
    }
    private final static JavaTimeModule timeModule = new JavaTimeModule();
    static {
        DateTimeFormatter dateFormat = DateTimeFormatter.ofPattern("MM-dd");
        MonthDayDeserializer dateDeserializer = new MonthDayDeserializer(dateFormat);
        timeModule.addDeserializer(MonthDay.class, dateDeserializer);
    }

    @Override
    public List<Task> getTaskList(Date start, Date end) throws IOException{
        List<Task> taskList = Collections.emptyList();

        if (dataSource != null) {
            ObjectMapper mapper = new ObjectMapper();
            mapper.registerModule(timeModule);

            taskList = mapper.readerForListOf(Task.class).readValue(dataSource);
        }

        return taskList;
    }

    @Override
    public void saveTask(Task task) throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        if(dataSource != null) {
            mapper.writeValue(new File(dataSource.getFile()), task);
        } else {
            mapper.writeValue(new File("taskdb.json"),task);
        }
    }

    @Override
    public void removeTask(Task task) throws IOException {
        List<Task> taskList;

        if(dataSource != null) {
            ObjectMapper mapper = new ObjectMapper();
            taskList = mapper.readerForListOf(Task.class).readValue(dataSource);
            for (int index = 0; index < taskList.size(); index++){
                if(task.equals(taskList.get(index))){
                    taskList.remove(task);
                    break;
                }
            }
        }
    }
}
